package com.dcsoft.system.service.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import com.dcsoft.common.core.utils.DateUtils;
import com.dcsoft.common.core.utils.SpringUtils;
import com.dcsoft.common.core.utils.StringUtils;
import com.dcsoft.system.domain.vo.TreeSelect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.dcsoft.system.mapper.SysSpaceMapper;
import com.dcsoft.system.domain.SysSpace;
import com.dcsoft.system.service.ISysSpaceService;

/**
 * 空间管理Service业务层处理
 *
 * @author nichun
 * @date 2023-03-08
 */
@Service
public class SysSpaceServiceImpl implements ISysSpaceService
{
    @Autowired
    private SysSpaceMapper sysSpaceMapper;

    /**
     * 查询空间管理
     *
     * @param id 空间管理主键
     * @return 空间管理
     */
    @Override
    public SysSpace selectSysSpaceById(Long id)
    {
        return sysSpaceMapper.selectSysSpaceById(id);
    }

    /**
     * 查询空间管理列表
     *
     * @param sysSpace 空间管理
     * @return 空间管理
     */
    @Override
    public List<SysSpace> selectSysSpaceList(SysSpace sysSpace)
    {
        return sysSpaceMapper.selectSysSpaceList(sysSpace);
    }

    /**
     * 新增空间管理
     *
     * @param sysSpace 空间管理
     * @return 结果
     */
    @Override
    public int insertSysSpace(SysSpace sysSpace)
    {
        sysSpace.setCreateTime(DateUtils.getNowDate());
        return sysSpaceMapper.insertSysSpace(sysSpace);
    }

    /**
     * 修改空间管理
     *
     * @param sysSpace 空间管理
     * @return 结果
     */
    @Override
    public int updateSysSpace(SysSpace sysSpace)
    {
        sysSpace.setUpdateTime(DateUtils.getNowDate());
        return sysSpaceMapper.updateSysSpace(sysSpace);
    }

    /**
     * 批量删除空间管理
     *
     * @param ids 需要删除的空间管理主键
     * @return 结果
     */
    @Override
    public int deleteSysSpaceByIds(Long[] ids)
    {
        return sysSpaceMapper.deleteSysSpaceByIds(ids);
    }

    /**
     * 删除空间管理信息
     *
     * @param id 空间管理主键
     * @return 结果
     */
    @Override
    public int deleteSysSpaceById(Long id)
    {
        return sysSpaceMapper.deleteSysSpaceById(id);
    }

    /**
     * 查询部门树结构信息
     *
     * @param space 空间信息
     * @return 部门树信息集合
     */
    @Override
    public List<TreeSelect> selectSpaceTreeList(SysSpace space)
    {
        List<SysSpace> spaces = sysSpaceMapper.selectSysSpaceList(space);
        return buildSpaceTreeSelect(spaces);
    }


    /**
     * 构建前端所需要下拉树结构
     *
     * @param spaces 空间列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildSpaceTreeSelect(List<SysSpace> spaces)
    {
        List<SysSpace> spaceTrees = buildSpaceTree(spaces);
        return spaceTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 构建前端所需要树结构
     *
     * @param spaces 空间列表
     * @return 树结构列表
     */
    @Override
    public List<SysSpace> buildSpaceTree(List<SysSpace> spaces)
    {
        List<SysSpace> returnList = new ArrayList<SysSpace>();
        List<Long> tempList = spaces.stream().map(SysSpace::getId).collect(Collectors.toList());
        for (SysSpace space : spaces)
        {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(space.getParentId()))
            {
                recursionFn(spaces, space);
                returnList.add(space);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = spaces;
        }
        return returnList;
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<SysSpace> list, SysSpace t)
    {
        // 得到子节点列表
        List<SysSpace> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysSpace tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysSpace> getChildList(List<SysSpace> list, SysSpace t)
    {
        List<SysSpace> tlist = new ArrayList<SysSpace>();
        Iterator<SysSpace> it = list.iterator();
        while (it.hasNext())
        {
            SysSpace n = (SysSpace) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysSpace> list, SysSpace t)
    {
        return getChildList(list, t).size() > 0 ? true : false;
    }

}
